clear all;
clc;
close all;

%% UCA settings
Nt = 600;
fc = 30e9;
c = 3e8;
lambda = c/fc;
d = lambda/2;
r_radius = Nt*d/2/pi; % half-wavelength spacing
D_fres = (2*r_radius)^(4/3)/2/(lambda)^(1/3);

%% plot the beam gain over distance
theta_0 = pi/3;
r_0 = 15;
phi_0 = 0;
[at, at_app] = generate_UCA_3D(fc, Nt, r_radius, r_0, theta_0, phi_0);

num_r = 4000;
r_list = linspace(D_fres*1.2, 50, num_r); %D_fres
g_r = zeros(1, num_r);
g_r_app = zeros(1, num_r);
g_r_est = zeros(1, num_r);
g_real_gain = zeros(1, num_r);
for idx_r = 1:num_r
    r_iter = r_list(idx_r);
    [h0, h0_app] = generate_UCA_3D(fc, Nt, r_radius, r_iter, theta_0, phi_0);
    g_r(idx_r)=  abs(at*h0');
    g_r_app(idx_r)=  abs(at_app*h0_app');
    g_real_gain(idx_r)=  abs(h0*h0_app');
    
    xi = pi*r_radius^2*sin(theta_0)^2/lambda/2*(1/(r_iter)-1/r_0);
    g_r_est(idx_r) = abs(besselj(0, xi));
end

C = linspecer(6);
plot_line = 0:0.01:1;
figure;
hold on;
plot(r_list, g_r,'-','LineWidth',1.5,'color', C(1,:));
plot(r_list, g_r_est,'--','LineWidth',2.5,'color', C(2,:));
legend('Accurate Beamforming Gain', 'Estimated Beamforming Gain with (7)',...
    'Location','SouthEast');
xlabel('Distance of user r (m)');
ylabel('Normalized beamforming gain');
grid on;
box on;