clear all;
clc;
close all;

%% parameters
Nt_max = 1200;
M = 20;
fc = 30e9;
c = 3e8;
lambda = c/fc;
d = lambda/2;
C = linspecer(6);

r_radius_max = Nt_max*d/2/pi; % half-wavelength spacing
r_radius_list = zeros(1, M);
Nt_list = zeros(1, M);
for i_m = 1:M
    r_radius_list(i_m) = r_radius_max/M*i_m;
    Nt_list(i_m) = Nt_max/M*i_m;
end
Nt_total = sum(Nt_list);

SNR_dB = 25;
SNR = 10.^(SNR_dB/10.);

%% random user locations
user_list = 1:7;
num_user = length(user_list);

rho_min = 4;
rho_max = 40;
rand_iter = 1200;
search_step = 1;
loc_list = rho_min:search_step:rho_max;

sum_rate_rand_ave = zeros(1, num_user);
theta0 = 0;
phi0 = 0;
for i_user = 1:num_user+1
    K = i_user;
    R_n_rand = zeros(1, K);

    sum_rate_rand = zeros(rand_iter, K);
    for i_rand_iter = 1:rand_iter
        loc_idx = randperm(length(loc_list), i_user);
        loc = loc_list(loc_idx);
        At_rand = zeros(Nt_total, i_user);
        for i_rand = 1:i_user
            At_rand(:,i_rand) = ...
            generate_con_UCA_3D(fc, Nt_list, r_radius_list, loc(i_rand), theta0, phi0);
        end
        cov_matrix_rand = (At_rand'*At_rand);
        inv_cov_diag_rand = diag(inv(cov_matrix_rand));
        H_e_rand = At_rand'*At_rand;
        power_alloc_rand = diag(1./sqrt(inv_cov_diag_rand));
        H_pinv_rand = H_e_rand'/(H_e_rand*H_e_rand')*power_alloc_rand/sqrt(K);
        gain_rand = abs(At_rand'*At_rand*H_pinv_rand)^2;

        for n = 1:K
            R_n_rand(n) = log2(1+SNR/K*(gain_rand(n,n)));
        end
        sum_rate_rand(i_rand_iter,i_user) = sum(R_n_rand);
    end
    sum_rate_rand_ave(i_user) = mean(sum_rate_rand(:,i_user));
end

%% optimal orthogonal users
% codebook
r_min = 4;
num_beam_distance = floor(r_radius_max^2/2/lambda/r_min);
dis_list_tmp = zeros(1, num_beam_distance);
for i_dis = 1:num_beam_distance
    if i_dis == 1
        dis_list_tmp(i_dis) = 10000;
    else
        dis_list_tmp(i_dis) = r_radius_max^2/2/lambda/(i_dis-1);
    end
end
phi_0 = 0;
phi_list_dis = phi_0*ones(1, num_beam_distance);
theta_0 = 0;
theta_list_dis = theta_0*ones(1, num_beam_distance);

[beam_near, ~] =...
    generate_con_UCA_3D(fc, Nt_list, r_radius_list, dis_list_tmp, theta_list_dis, phi_list_dis);

sum_rate= zeros(1, num_user);
for i_user = 1:num_user+1
    if i_user == 1
        K = i_user;
        w_near_distance = beam_near(1,:).';
        cov_matrix_dis = (w_near_distance'*w_near_distance);
        
        gain = cov_matrix_dis;
        R_n = log2(1+SNR*gain);
        sum_rate(i_user) = sum(R_n);
    else
        K = i_user;
        At = beam_near(1:K,:).';
        cov_matrix = (At'*At);
        inv_cov_diag = diag(inv(cov_matrix));
        H_e = At'*At;
        power_alloc = diag(1./sqrt(inv_cov_diag));
        H_pinv = H_e'/(H_e*H_e')*power_alloc/sqrt(K);
        gain = abs(At'*At*H_pinv)^2;
        for n = 1:K
            R_n(n) = log2(1+SNR/K*(gain(n,n)));
        end
        sum_rate(i_user) = sum(R_n);
    end
    
end

x_plot = 1:1:user_list(end)+1;
figure;
hold on;
plot(x_plot,sum_rate_rand_ave(1)*ones(1, user_list(end)+1),'k--','Linewidth',1.5);
plot(x_plot,sum_rate_rand_ave,'-^','Linewidth',1.5,'color', C(1,:));
plot(x_plot,sum_rate,'-*','Linewidth',1.5,'color', C(2,:));
xlabel('Number of users');
ylabel('Spectrum efficiency (bps/Hz)');
legend('Far-field BF', 'Random users with near-field BF',...
    'Optimal users with near-field BF','Location','NorthWest');
grid on;
box on;